/*
 * Copyright (c) 2013-2016 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * PROPRIETARY/CONFIDENTIAL
 *
 * This software is the confidential and proprietary information of
 * SAMSUNG ELECTRONICS ("Confidential Information").
 * You shall not disclose such Confidential Information and shall
 * use it only in accordance with the terms of the license agreement
 * you entered into with SAMSUNG ELECTRONICS.
 * SAMSUNG make no representations or warranties about the suitability
 * of the software, either express or implied, including but not
 * limited to the implied warranties of merchantability, fitness for
 * a particular purpose, or non-infringement.
 * SAMSUNG shall not be liable for any damages suffered by licensee as
 * a result of using, modifying or distributing this software or its derivatives.
 */

#ifndef __SAP_MESSAGE_EXCHANGE_H__
#define __SAP_MESSAGE_EXCHANGE_H__
/**
 * @file            sap_message_exchange.h
 * @brief           SAP Connection-less SDK interface API
 * @addtogroup      SAMSUNG_ACCESSORY_PROTOCOL
 * @{
 */

#ifdef __cplusplus
extern "C" {
#endif


/**
 * @ingroup         SAMSUNG_ACCESSORY_PROTOCOL
 * @brief           The structure type for the peer agent handle
 * @since_tizen     2.3.1
 */
typedef struct sap_peer_agent_s *sap_peer_agent_h;

/**
 * @ingroup         SAMSUNG_ACCESSORY_PROTOCOL
 * @brief           This callback function gets triggered when the application receives data from peer by
 *                  connection-less data transfer mode.
 * @since_tizen     2.3.2
 *
 * @param[in]       peer_agent          The handle of the peer_agent for which data has been received
 * @param[in]       payload_length      The length of payload data.
 * @param[in]       buffer              The data buffer that has been received
 * @param[in]       user_data           The user data passed from the register function
 */
typedef  void (*sap_agent_data_received_cb)(sap_peer_agent_h peer_agent,
					    unsigned int payload_length,
					    void *buffer,
					    void *user_data);

/**
 * @ingroup         SAMSUNG_ACCESSORY_PROTOCOL
 *
 * @brief           To register a callback function so as to receive data through connection less layer from a remote peer.
 *
 * @since_tizen     2.3.2
 *
 * @param[in]       agent           Agent handle which was created using #sap_agent_create() and
 *                                  updated in #sap_agent_initialized_cb() callback.
 * @param[in]       callback        The callback function to register
 * @param[in]       user_data       The user data to be passed to the callback function
 *
 * @return          #sap_result_e
 *
 * @privlevel       public
 * @privilege       %http://developer.samsung.com/tizen/privilege/accessoryprotocol
 *
 * @see             #sap_agent_data_received_cb()
 *
 */
int sap_agent_set_data_received_cb(sap_agent_h agent,
				   sap_agent_data_received_cb callback,
				   void *user_data);

/**
 * @ingroup         SAMSUNG_ACCESSORY_PROTOCOL
 * @brief           Enumerations for the result of sap connection less data transfer operation.
 * @since_tizen     2.3.2
 */
typedef enum {
	/**  Message feature not supported by remote device */
	SAP_CONNECTIONLESS_TRANSFER_ERROR_REMOTE_DEVICE_NOT_SUPPORTED = -8,
	/**  Message feature not enabled in remote app */
	SAP_CONNECTIONLESS_TRANSFER_ERROR_PEER_AGENT_NOT_SUPPORTED,
	/**  Message feature not enabled in local app */
	SAP_CONNECTIONLESS_TRANSFER_ERROR_LOCAL_AGENT_NOT_SUPPORTED,
	/**  Invalid arguments provided*/
	SAP_CONNECTIONLESS_TRANSFER_INVALID_PARAMETERS,
	/**  Max limit for data queue reached. Can handle new send requests after processing existing requests. */
	SAP_CONNECTIONLESS_TRANSFER_MAX_REQUEST_LIMIT_REACHED,
	/**  Connection Less SDK Input Output Error. This occurs when there is no space left in the device.*/
	SAP_CONNECTIONLESS_TRANSFER_IO_ERROR,
	/**  Transfer is bound to Max payload size of 2MB*/
	SAP_CONNECTIONLESS_TRANSFER_EXCEEDED_MAX_APDU_LEN,
	/**  Wrong peer agent handle provided */
	SAP_CONNECTIONLESS_TRANSFER_INVALID_PEER_AGENT_HANDLE
} sap_connectionless_transfer_result_e;

/**
 * @ingroup         SAMSUNG_ACCESSORY_PROTOCOL
 * @brief           Enumerations for the status of sap connection less data transfer operation.
 * @since_tizen     2.3.2
 */
typedef enum {
	/**  Successful connectionless data transfer*/
	SAP_CONNECTIONLESS_TRANSFER_ERROR_NONE = 0,
	/**  Remote Peer app is not alive */
	SAP_CONNECTIONLESS_TRANSFER_PEER_APP_ERROR = 1,
	/**  Connection less Data transfer failed due to network error*/
	SAP_CONNECTIONLESS_TRANSFER_NETWORK_ERROR = 2,
	/**  Connection less Data transfer failed due to Error in SAP framework*/
	SAP_CONNECTIONLESS_TRANSFER_FRAMEWORK_INTERNAL_ERROR = 3,
	/**  Unknown error occurred while sending data over connection less layer*/
	SAP_CONNECTIONLESS_TRANSFER_UNKNOWN_ERROR = 4
} sap_connectionless_transfer_status_e;

/**
 * @ingroup         SAMSUNG_ACCESSORY_PROTOCOL
 * @brief           The app will be notified about its data transfer transaction (done using
 *                  #sap_peer_agent_send_data()) via this callback
 *
 * @since_tizen     2.3.2
 *
 * @param[in]       peer_agent              The handle for the peer_agent
 * @param[in]       status                  Status code value is one of the enums defined in
 *                                          #sap_connectionless_transfer_status_e.
 * @param[in]       transaction_id          A unique transaction id which was returned in
 *                                          #sap_peer_agent_send_data().
 * @param[in]       user_data               The user data passed from the register callback function
 *
 * @see             #sap_agent_set_data_received_cb()
 *
 */
typedef  void (*sap_peer_agent_message_delivery_status_cb)(sap_peer_agent_h peer_agent_h, int transaction_id, sap_connectionless_transfer_status_e status, void *user_data);

/**
 * @ingroup         SAMSUNG_ACCESSORY_PROTOCOL
 * @brief           Send data to the specified peer agent without making a service connection.
 *                  Status of the message delivery will be notified in #sap_peer_agent_message_delivery_status_cb() callback.
 *
 * @since_tizen     2.3.2
 *
 * @param[in]       peer_agent                  The handle for the peer_agent
 * @param[in]       buffer                      The data buffer to be sent
 * @param[in]       payload_length              The length of payload data
 * @param[in]       encrypt_data                If set to a positive value, SAP framework will encrypt the data
 *                                              before sending.
                                                If set to 0, data will not be encrypted.
 * @param[in]       callback                    The callback function to receive message delivery status.
 * @param[in]       user_data                   The user data to be passed to the callback function
 *
 * @return          Returns a positive unique transaction id for the current transaction, same will be reflected
 *                  back in
                    #sap_peer_agent_message_delivery_status_cb() callback with its status defined in enum #sap_connectionless_transfer_status_e.
                    For negative return values refer status codes defined in enum #sap_connectionless_transfer_result_e.
 *
 * @privlevel       public
 * @privilege       %http://developer.samsung.com/tizen/privilege/accessoryprotocol
 *
 * @pre             App should receive a valid peer agent handle in callback #sap_peer_agent_updated_cb(),
 *                  by calling #sap_agent_find_peer_agent()
 *
 * @see             #sap_peer_agent_message_delivery_status_cb()
 */
int sap_peer_agent_send_data(sap_peer_agent_h peer_agent,
			     unsigned char *buffer,
			     unsigned int payload_length,
			     int encrypt_data,
			     sap_peer_agent_message_delivery_status_cb callback,
			     void *user_data);

#ifdef __cplusplus
}
#endif

/**
 * @}
 */
#endif /* __SAP_MESSAGE_EXCHANGE_H__ */

